/*

The infolock Challenge:

<infolock> code me a simple linked list, the list will store the file names in a 
directory then print them out "a.out /root" and give me a dir listing
of /root using a linked list.

*/


#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>

struct listNode {
	char filename[256];
	struct listNode *next;
};

typedef struct listNode NODE;
typedef NODE *NODEPTR;

void listDir(char *);
int addNode(NODEPTR *, char *);
void freeList(NODEPTR *);
void usage();

int main(int argc, char *argv[])
{

	if(argc != 2) {
		usage();
	}

	listDir(argv[1]);

	return 0;
}


void listDir(char *path)
{
	
	NODEPTR dirContents = NULL;
	char fullPath[257];
	DIR *dirName;
	struct dirent *dirEntry;

	strncpy(fullPath,path,254);
	fullPath[255] = '\0';
	
	if(fullPath[strlen(fullPath)-1] != '/') {
		strcat(fullPath,"/");		
	}

	if((dirName = opendir(fullPath)) == NULL) {
		printf("cannot open directory %s\n",fullPath);
		return;
	}
	
	while((dirEntry = readdir(dirName)) != NULL) {
		if(!addNode(&dirContents,dirEntry->d_name)) {
			printf("error adding %s to list\n",dirEntry->d_name);
			return;
		}
	}

	printf("\nDirectory Listing for %s:\n\n",fullPath);

	while(dirContents != NULL) {
		printf("%s%s\n",fullPath,dirContents->filename);
		dirContents = dirContents->next;
	}

	freeList(&dirContents);
	
	return;

}	

void freeList(NODEPTR *top)
{

	NODEPTR current;
	
	if(*top == NULL)
		return;
	
	current = *top;
	free(current);
	
	return;
	
}
	
	

int addNode(NODEPTR *top, char *name)
{

	NODEPTR new, current;

	(NODEPTR)new = malloc(sizeof(NODE));
	if(new == NULL) {
		printf("out of memory\n");
		return 0;
	}
	new->next = NULL;
	strncpy(new->filename, name, 254);
	new->filename[255] = '\0';

	current = *top;

	if(current == NULL) 
		*top = new;	
	else {
		while(current->next != NULL) 
			current = current->next;
		current->next = new;
	}
	
	return 1;

}

void usage()
{

	printf("Syntax: a.out <path>\n");
	exit(0);

}